<?php

namespace Drupal\flockler\Plugin\Block;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a 'FlocklerBlock' block.
 *
 * @Block(
 *  id = "flockler_block",
 *  admin_label = @Translation("Flockler block"),
 * )
 */
class FlocklerBlock extends BlockBase {

  /**
   * Guzzle Http Client.
   *
   * @var GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form['url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Feed URL'),
      '#description' => $this->t('The URL of the Flockler feed'),
      '#default_value' => $this->configuration['url'],
      '#maxlength' => 64,
      '#size' => 64,
      '#weight' => '0',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->configuration['url'] = $form_state->getValue('url');
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $build = [];
    $build['content'] = $this->fetchFeed();
    $build['#attributes'] = [
      'class' => [
        'flockler-articles',
      ],
    ];
    return $build;
  }

  private function fetchFeed() {
    $url = $this->configuration['url'];

    // TODO: use dependency injection.
    $response = \Drupal::httpClient()->get($url);
    $body = $response->getBody()->__toString();
    $jsonObject = Json::decode($body, true);

    // TODO: cache contexts.
    return $this->formatFeed($jsonObject);
  }

  private function formatFeed($jsonObject) {
    $render = [];

    $items = [];
    if (!empty($jsonObject)) {
      $render = [
        '#attributes' => [
          'class' => [
            'flockler-articles',
          ],
        ],
      ];

      foreach ($jsonObject['articles'] as $article) {
        $render[] = $this->formatArticle($article);
      }
    }
    return $render;
  }

  /**
   * Prepare an article
   *
   * @param array $article
   *   The JSON representation of the article in the feed.
   *
   * @return array
   *   A render array.
   */
  private function formatArticle($article) {
    $render = [];

    switch ($article['type']) {
      case 'tweet':
        $render = $this->formatTweet($article);
        break;
      case 'instagram':
        $render = $this->formatInstagram($article);
        break;
    }

    return $render;
  }

  /**
   * Prepare a render array for a tweet.
   *
   * @param array $article
   *   The JSON representation of the tweet.
   *
   * @return array
   *   A render array for the tweet.
   */
  private function formatTweet($article) {

    $tweet = $article['attachments']['tweet'];

    $render = [
      '#theme' => 'flockler_article',
      '#content' => [
        'type' => 'tweet',
        'profile_image' => $tweet['profile_image_url'],
        'profile_name' => $tweet['screen_name'],
        'profile_link' => 'https://twitter.com/' . $tweet['name'],
        'text' => $tweet['text'],
        'image' => $tweet['media_url'],
        'link' => 'https://twitter.com/' . $tweet['name'] . '/status/' . $tweet['tweet_id_str'],
      ],
    ];
    return $render;
  }

  /**
   * Prepare a render array for an instagram post.
   *
   * @param array $article
   *   The JSON representation of the post.
   *
   * @return array
   *   A render array for the post.
   */
  private function formatInstagram($article) {
    $post = $article['attachments']['instagram_item'];

    $render = [
      '#theme' => 'flockler_article',
      '#content' => [
        'type' => 'instagram',
        'text' => $post['caption'],
        'profile_image' => $post['profile_picture'],
        'profile_name' => $post['user_full_name'],
        'profile_link' => 'https://www.instagram.com/' . $post['username'],
        'link' => $post['link'],
        'image' => $article['cover_url'],
      ],
    ];
    return $render;
  }
}
